@@ -14,20 +14,19 @@
  * limitations under the License.
  */
 
-use crate::attributes::MTestAttributes;
-use crate::TResult;
-use crate::TestGeneratorError;
+use crate::attributes::{MTestAttributes, ServiceDescription};
 use crate::marine_test;
-use crate::marine_test::config_utils;
+use crate::marine_test::{config_utils, token_stream_generator};
+use crate::TestGeneratorError;
+use crate::TResult;
+
+use std::path::PathBuf;
 
-use fluence_app_service::TomlAppServiceConfig;
 use proc_macro2::TokenStream;
 use quote::quote;
 use quote::ToTokens;
-
-use std::path::Path;
-use std::path::PathBuf;
 use syn::FnArg;
+use std::collections::HashMap;
 
 /// Generates glue code for tests.
 /// F.e. for this test for the greeting service
@@ -116,23 +115,35 @@ use syn::FnArg;
 pub(super) fn generate_test_glue_code(
     func_item: syn::ItemFn,
     attrs: MTestAttributes,
-    file_path: PathBuf,
+    test_file_path: PathBuf,
 ) -> TResult<TokenStream> {
-    let config_path = file_path.join(&attrs.config_path);
+    match attrs {
+        MTestAttributes::MultipleServices(services) => {
+            generate_test_glue_code_multiple_services(func_item, services, test_file_path)
+        }
+        MTestAttributes::SingleService(service) => {
+            generate_test_glue_code_single_service(func_item, service, test_file_path)
+        }
+    }
+}
 
-    let marine_config = TomlAppServiceConfig::load(&config_path)?;
-    let modules_dir = match config_utils::resolve_modules_dir(&marine_config, attrs.modules_dir) {
-        Some(modules_dir) => modules_dir,
-        None => return Err(TestGeneratorError::ModulesDirUnspecified),
-    };
-
-    let app_service_ctor = generate_app_service_ctor(&attrs.config_path, &modules_dir)?;
-    let modules_dir = file_path.join(modules_dir);
+fn generate_test_glue_code_single_service(
+    func_item: syn::ItemFn,
+    service: ServiceDescription,
+    test_file_path: PathBuf,
+) -> TResult<TokenStream> {
+    let config_wrapper =
+        config_utils::load_config(&service.config_path, service.modules_dir, &test_file_path)?;
+    let modules_dir_test_relative = test_file_path.join(&config_wrapper.resolved_modules_dir);
     let module_interfaces =
-        marine_test::config_utils::collect_modules(&marine_config, modules_dir)?;
-    let linked_modules = marine_test::modules_linker::link_modules(&module_interfaces)?;
+        config_utils::collect_modules(&config_wrapper.config, &modules_dir_test_relative)?;
+    let linked_modules = marine_test::modules_linker::link_modules(
+        module_interfaces
+            .iter()
+            .map(|module| (module.name, &module.interface)),
+    )?;
 
-    let module_definitions = marine_test::module_generator::generate_module_definitions(
+    let module_definitions = token_stream_generator::generate_module_definitions(
         module_interfaces.iter(),
         &linked_modules,
     )?;
@@ -143,7 +154,10 @@ pub(super) fn generate_test_glue_code(
     let inputs = &signature.inputs;
     let arg_names = generate_arg_names(inputs.iter())?;
     let module_ctors = generate_module_ctors(inputs.iter())?;
-
+    let app_service_ctor = token_stream_generator::generate_app_service_ctor(
+        &service.config_path,
+        &config_wrapper.resolved_modules_dir,
+    )?;
     let glue_code = quote! {
         #[test]
         fn #name() {
@@ -170,62 +184,34 @@ pub(super) fn generate_test_glue_code(
     Ok(glue_code)
 }
 
-fn generate_app_service_ctor(config_path: &str, modules_dir: &Path) -> TResult<TokenStream> {
-    let modules_dir = modules_dir
-        .to_str()
-        .ok_or_else(|| TestGeneratorError::InvalidUTF8Path(modules_dir.to_path_buf()))?;
+fn generate_test_glue_code_multiple_services(
+    func_item: syn::ItemFn,
+    services: HashMap<String, ServiceDescription>,
+    test_file_path: PathBuf,
+) -> TResult<TokenStream> {
+    let service_definitions =
+        token_stream_generator::generate_service_definitions(services, &test_file_path)?;
 
-    let service_ctor = quote! {
-        let tmp_dir = std::env::temp_dir();
-        let service_id = marine_rs_sdk_test::internal::Uuid::new_v4().to_string();
-
-        let tmp_dir = tmp_dir.join(&service_id);
-        let tmp_dir = tmp_dir.to_string_lossy().to_string();
-        std::fs::create_dir(&tmp_dir).expect("can't create a directory for service in tmp");
-
-        let mut module_path = std::path::PathBuf::from(env!("CARGO_MANIFEST_DIR"));
-        let mut file_path = std::path::Path::new(file!()).components();
-
-        let mut truncated_file_path = Vec::new();
-        loop {
-            if module_path.ends_with(file_path.as_path()) {
-                break;
+    let original_block = func_item.block;
+    let signature = func_item.sig;
+    let name = &signature.ident;
+    let glue_code = quote! {
+        #[test]
+        fn #name() {
+            // definitions for services specified in attributes
+            pub mod marine_test_env {
+              #(#service_definitions)*
             }
 
-            let (file_path_, remainder) = match file_path.next_back().and_then(|p| match p {
-                std::path::Component::Normal(_) | std::path::Component::CurDir | std::path::Component::ParentDir => Some((file_path, p)),
-                _ => None,
-            }) {
-                Some(t) => t,
-                None => break,
-            };
-            file_path = file_path_;
+            fn test_func() {
+               #original_block
+            }
 
-            truncated_file_path.push(remainder);
+            test_func()
         }
-
-        for path in truncated_file_path.iter().rev() {
-            module_path.push(path);
-        }
-
-        let _ = module_path.pop();
-
-        let config_path = module_path.join(#config_path);
-        let modules_dir = module_path.join(#modules_dir);
-        let modules_dir = modules_dir.to_str().expect("modules_dir contains invalid UTF8 string");
-
-        let mut __m_generated_marine_config = marine_rs_sdk_test::internal::TomlAppServiceConfig::load(&config_path)
-            .unwrap_or_else(|e| panic!("app service config located at `{:?}` can't be loaded: {}", config_path, e));
-        __m_generated_marine_config.service_base_dir = Some(tmp_dir);
-        __m_generated_marine_config.toml_faas_config.modules_dir = Some(modules_dir.to_string());
-
-        let marine = marine_rs_sdk_test::internal::AppService::new_with_empty_facade(__m_generated_marine_config, service_id, std::collections::HashMap::new())
-            .unwrap_or_else(|e| panic!("app service can't be created: {}", e));
-
-        let marine = std::rc::Rc::new(std::cell::RefCell::new(marine));
     };
 
-    Ok(service_ctor)
+    Ok(glue_code)
 }
 
 fn generate_module_ctors<'inputs>(
