@@ -17,7 +17,7 @@
 mod config_utils;
 mod marine_test_impl;
 mod glue_code_generator;
-mod module_generator;
+mod token_stream_generator;
 mod utils;
 mod modules_linker;
 
